// ======================================================================
//
// MenuElement.h
// jeff grills
//
// copyright 1997 origin systems inc
//
// ======================================================================

#ifndef MENU_ELEMENT_H
#define MENU_ELEMENT_H

// ======================================================================

class Bitmap;
class Iff;
class StringFile;
class MenuResource;
class MenuResourceInteger;
class MenuResourceIntegerAnimation;

#include "Rect.h"
#include "IoWin.h"
#include "MenuResource.h"

// ======================================================================

enum MenuState
{
	MenuStateInactive, 
	MenuStateActivating,
	MenuStateActive,
	MenuStateSelected,
	MenuStateInactivating
};

// ----------------------------------------------------------------------

class MenuElement : public IoWin
{
protected:

	enum TabStop
	{
		TabOn,
		TabOff,
		TabForceOn,
		TabForceOff
	};

protected:

	char             *menuId;
	MenuState         menuState;
	Rect              menuRect;
	MenuResource     *menuResource;
	MenuElement      *parent;
	MenuElement      *firstChild;
	MenuElement      *nextSibling;

	// these should only get used in the "root" of the menu (ie: parent != NULL)
	Point            *tabStops;
	int              numTabStops;
	int              curTabStop;
	TabStop          tabStop;

protected:

	virtual MenuResource *findSharedResource(MenuResource::Type type, const char *id);
	void                 addTabStop(int x, int y);

private:

	MenuElement(void);
	MenuElement(const MenuElement &);
	MenuElement &operator =(const MenuElement &);

public:

	static MenuElement *create(MenuElement *parent, int maxWidth, int maxHeight, Iff *iff);

public:

	MenuElement(MenuElement *parent, int maxWidth, int maxHeight, Iff *iff=NULL);
	virtual ~MenuElement(void);

	void load(Iff *iff);
	void move(int deltaX, int deltaY);

	MenuElement  *getNextSibling(void);
	const Rect   &getRect(void) const;
	const char   *getId(void) const;
	void          getAbsoluteRect(Rect *rect) const;

	// IoWin routines
	virtual IoResult event(IoEventStruct *event);
	virtual IoResult update(void);
	virtual void draw(void) const;

	virtual void         menuEvent(int id, MenuElement *whom);
	virtual void         menuDraw(int x, int y) const;

	Bitmap                       *findSharedBitmap(const char *id);
	Font                         *findSharedFont(const char *id);
	MenuResourceInteger          *findSharedInteger(const char *id);
	MenuResourceIntegerAnimation *findSharedIntegerAnimation(const char *id);
	StringFile                   *findSharedLanguage(const char *id);
	MenuElement                  *findMenu(const char *findId, bool required=true);
};

// ======================================================================

inline const Rect &MenuElement::getRect(void) const
{
	return menuRect;
}

// ----------------------------------------------------------------------

inline const char *MenuElement::getId(void) const
{
	return menuId;
}

// ----------------------------------------------------------------------

inline void MenuElement::move(int deltaX, int deltaY)
{
	menuRect.x0 += deltaX;
	menuRect.y0 += deltaY;
	menuRect.x1 += deltaX;
	menuRect.y1 += deltaY;
}

// ----------------------------------------------------------------------

inline MenuElement *MenuElement::getNextSibling(void)
{
	return nextSibling;
}

// ======================================================================

#endif
